##############################################################################################################
# This script imports a polygon shapefile layer (.shp) containing training data for a supervised land cover
# classification.  A specified number of training pixels are sampled from each polygon (default=1).
#


##############################################################################################################
# USER INPUTS NEEDED
##############################################################################################################

# Step 1: Give path to folder with Landsat-processing R scripts
ScriptFolder <- "C:/Users/Grant/Desktop/SI_Postdoc/TNI_LCmapping/Analysis/Scripts/"

# Step 2: Give path to folder for temporary/output rasters
OutFolder <- "D:/GIS/Myanmar/LSAT/Tanintharyi/"

# Step 3: Give the filename for the DEM to use for topographic correction (same projection and resolution)
DEMFileName <- "D:/GIS/Myanmar/Elevation/tni_DEMSRTM_30m_NASA/SourceData/tni_DEMSRTM_30m_NASA_WGS84_utm.tif"

# Step 4: Specify base directory for Landsat imagery and an array giving folder paths to unzipped images
# Currently, this must be in the same format as below
baseDir <- "D:/GIS/Myanmar/LSAT"
imageDirArray <- matrix(c("129_52","LC81290522016071LGN00.tar","LC81290522016071LGN00",
                          "130_50","LC81300502016046LGN00.tar","LC81300502016046LGN00",
                          "130_51","LC81300512016046LGN00.tar","LC81300512016046LGN00",
                          "130_52","LC81300522016078LGN00.tar","LC81300522016078LGN00",
                          "130_53","LC81300532016078LGN00.tar","LC81300532016078LGN00",
                          "131_50","LC81310502016069LGN00.tar","LC81310502016069LGN00",
                          "131_51","LC81310512016069LGN00.tar","LC81310512016069LGN00",
                          "131_52","LC81310522016069LGN00.tar","LC81310522016069LGN00"),ncol=3,byrow=T)

# Load necessary libraries
library(raster)
library(mvtnorm)
library(maptools)
library(caTools)
library(rgdal)

##############################################################################################################
# CLASSIFY IMAGE
##############################################################################################################

# Set Working Directory
setwd(OutFolder)

# Set Number of Classes in Training Data
nclass=17

# Read in Training Data
TrainingPolygons <- readOGR(dsn="D:/GIS/Myanmar/GroundTruthing/GE",layer="Merged_2016")
TrainingPolygons1 <- spTransform(TrainingPolygons,crs("+proj=longlat +datum=WGS84"))  # Reproject

# Read in Region Boundary
TNI_Boundary <- readOGR(dsn="D:/GIS/Myanmar/MIMU/state", layer="state")
TNI_Boundary <- spTransform(TNI_Boundary,crs("+proj=longlat +datum=WGS84"))           # Reproject

# Limit training data to polygons inside the region of interest
TrainingPolygons1 <- intersect(TrainingPolygons1,TNI_Boundary)

# Calculate minimum number of training polygons per class
NT <- 75

# Randomly select NT number of training polygons for each class
IN <- matrix(NA,nrow=NT,ncol=nclass)
for (i in unique(TrainingPolygons1@data$id)){
  IN[,i] <- sample(which(TrainingPolygons1@data$id==i),size=NT,replace=T)
}
TrainingPolygons1 <- TrainingPolygons1[as.numeric(IN),]

# Divide sample into training data and validation data
train_rows <- sample.split(TrainingPolygons1@data[,2], SplitRatio=0.66)
TrainingPolygons <- TrainingPolygons1[train_rows,]
ValidationPolygons  <- TrainingPolygons1[!(train_rows),]

# Optional - Can change n= argument to use multiple pixels from each polygon as training data
Locs <- do.call(rbind, sapply(slot(TrainingPolygons, "polygons"), spsample, n=1, type="random"))
DF <- as.data.frame(Locs@coords)
DF$id <- rep(TrainingPolygons@data$id,each=1)
Training <- DF

DeleteCashew <- which(Training$id==14)
Training <- Training[-DeleteCashew,]
Training$id[which(Training$id>14)] <- Training$id[which(Training$id>14)]-1

Training <- SpatialPointsDataFrame(cbind(Training$x,Training$y),data = data.frame(Class=Training$id))
proj4string(Training) <- CRS("+proj=longlat + ellps=WGS84")
Training <-spTransform(Training,CRS=crs("+proj=utm +zone=47 +datum=WGS84 +units=m +no_defs +ellps=WGS84 +towgs84=0,0,0"))
Training <- crop(Training,raster("m_B2.tif"))

plot(raster("m_B2.tif"))
plot(Training,pch=19,add=T)

C <- cbind(extract(x=raster("m_B2.tif"),y=Training),
           extract(x=raster("m_B3.tif"),y=Training),
           extract(x=raster("m_B4.tif"),y=Training),
           extract(x=raster("m_B5.tif"),y=Training),
           extract(x=raster("m_B6.tif"),y=Training),
           extract(x=raster("m_B7.tif"),y=Training),
           extract(x=raster("TPI.tif"),y=Training),
           extract(x=raster("DEM.tif"),y=Training))

# Extract index positions of points to classify (training data + points.to.classify)
Template <- raster("m_B2.tif")

ndim=dim(C)[2]
means <- sd <- numeric()  # initialize vectors to store means and standard deviations for each band
for (i in 1:ndim){
  means[i] <- mean(C[,i],na.rm=T) # calculate mean for band
  sd[i] <- sd(C[,i],na.rm=T)      # calculate sd for band
  C[,i] <- (C[,i]-means[i])/sd[i]  # rescale (subtracting mean and dividing by sd)
}

notmissing <- which(!is.na(C[,1]))
C <- C[notmissing,]
Training <- Training[notmissing,]

require(RANN)
require(jagsUI)
require(MASS)
require(MCMCpack)

ndim=ncol(C)
nclass=16

sink("mix.txt")
cat("
    model {
    prob[1:nclass] ~ ddirich(alpha[]) # Assign priors to class membership probs.
    
    for(i in 1:nind){  # Assign to classes
    class[i] ~ dcat(prob[])
    }
    
    for(k in 1:nclass){  # Assign priors for mu/tau
    mu[k,1:ndim] ~ dmnorm(mu0[],M1[,])
    tau[k,1:ndim,1:ndim] ~ dwish(M2[,],ndim)
    sigma[k,1:ndim,1:ndim] <- inverse(tau[k,1:ndim,1:ndim])
    }
    
    for(i in 1:nind){
    C[i,1:ndim] ~ dmnorm(mu[class[i],],tau[class[i],,])
    } # Likelihood
    }
    ",fill = TRUE)
sink()

# Bundle data
mu0 = rep(0,ndim)
M1 = diag(0.01,ndim,ndim)  # Variance-covariance matrix for mv-normal priors for mus
M2 = diag(0.01,ndim,ndim)  # Prior for precision matrix
win.data <- list(C=C,nind=dim(C)[1],alpha=rep(1,nclass),M1=M1,M2=M2,mu0=mu0,
                 ndim=ndim,nclass=nclass,
                 class=Training@data$Class)

# Initial values
muk = array(numeric(nclass*ndim),c(nclass,ndim))
for(i in 1:nclass){
  muk[i,] = apply(C[which(Training@data$Class==i),],2,FUN=mean,na.rm=T)
}

tauk = array(numeric(nclass*ndim*ndim),c(nclass,ndim,ndim))

for(i in 1:nclass){
  tauk[i,,] = rwish(ndim,diag(1,ndim,ndim))
}

ClassI <- sample(x=1:nclass,size=dim(C)[1],replace=T)
ClassI[1:(dim(C)[1])] <- NA

inits= function(){list(mu = muk, tau = tauk, class = ClassI)}

# Parameters monitored
params <- c("mu","sigma","prob","class")

# Call JAGS from R (~ 1 min)
system.time(OUT <- jags(win.data, inits, params, codaOnly = "class", "mix.txt", parallel=TRUE, 
                        n.chains = 3,
                        n.thin = 5,
                        n.iter = 10000,
                        n.burnin = 5000))

ModMean <- OUT$mean$mu  # Estimated means for each band
ModVar <- OUT$mean$sigma   # Estimated variance-covariance matrix
SimsMean <- OUT$sims.list$mu
SimsVar <- OUT$sims.list$sigma

##############################################################################################################
MakeLayer <- function(x, mean, sigma, filename) {   # Function to write land cover raster based on trained model
  out <- raster(x)                                  # Template raster to fill with class values
  bs <- blockSize(out,minrows=1000)                 # For memory reasons, we will write the final raster in blocks
  out <- writeStart(out, filename, overwrite=TRUE)  # Start writing data to the raster

  for (i in 1:bs$n) {    # Loop through all raster cells in each block
    cat(paste0(i," of ", bs$n),"\n")   # Print the current block number to the console window to track progress
    CNew <- cbind(getValues(raster("m_B2.tif"),row=bs$row[i],nrows=bs$nrows[i]),  # Extract reflectance values for raster cells to CNew
                  getValues(raster("m_B3.tif"),row=bs$row[i],nrows=bs$nrows[i]),
                  getValues(raster("m_B4.tif"),row=bs$row[i],nrows=bs$nrows[i]),
                  getValues(raster("m_B5.tif"),row=bs$row[i],nrows=bs$nrows[i]),
                  getValues(raster("m_B6.tif"),row=bs$row[i],nrows=bs$nrows[i]),
                  getValues(raster("m_B7.tif"),row=bs$row[i],nrows=bs$nrows[i]),
                  getValues(raster("TPI.tif"),row=bs$row[i],nrows=bs$nrows[i]),
                  getValues(raster("DEM.tif"),row=bs$row[i],nrows=bs$nrows[i]))
    
    for (v in 1:ndim){   # For each column (columns=Landsat Bands, TPI, DEM) we rescale by the mean and sd for that variable
      CNew[,v] <- (CNew[,v]-means[v])/sd[v]  # rescale (subtracting mean and dividing by sd)
    }
    
    Vals <- matrix(NA,nrow=dim(CNew)[1], ncol=nclass)  # Create empty matrix to store probability densities for each possible class
    
    for (j in 1:nclass){   # Loop over possible classes
      Vals[,j] <- dmvnorm(CNew, mean=ModMean[j,], sigma=ModVar[j,,])  # For all raster cells, calculate probability density for class
    }
    
    Vals[is.na(Vals)] <- 0  # Replace NA values with 0
    Vals2 <- unlist(apply(Vals, 1, function(x) max(which(x==max(x)))))  # For all raster cells, find most-likely class

    out <- writeValues(out, Vals2, bs$row[i])  # Write those classes to the template raster
  }
  out <- writeStop(out)  # Stop writing data to raster
  return(out)  # Return the final raster
}

# Create final raster
s <- MakeLayer(raster("m_B2.tif"), mean=ModMean, sigma=ModVar, filename=paste0(OutFolder,"/LC_2016_GMLC2.tif"))[[1]]

# Overwrite cloud/no-data pixels with NA
s[which(is.na(raster("m_B2.tif")[]))] <- NA

# Write raster to file
writeRaster(s,filename=paste0(OutFolder,"/LC_2016_GMLC2.tif"),format="GTiff",overwrite=TRUE)

##############################################################################################################
MakeUncertaintyLayer <- function(x, mean, sigma, sims, filename) {  # Function to make raster of cell-level posterior probabilities
  out <- raster(x)                                   # Template raster to fill with posterior probability values
  bs <- blockSize(out,chunksize=1000000)             # For memory reasons, we will write the final raster in blocks
  out <- writeStart(out, filename, overwrite=TRUE)   # Start writing data to raster

  for (i in 1:bs$n) {   # Loop through all raster cells in each block
    cat(paste0(i," of ", bs$n),"\n")   # Print the current block number to the console window to track progress
    CNew <- cbind(getValues(raster("m_B2.tif"),row=bs$row[i],nrows=bs$nrows[i]),   # Extract reflectance values for raster cells to CNew
                  getValues(raster("m_B3.tif"),row=bs$row[i],nrows=bs$nrows[i]),
                  getValues(raster("m_B4.tif"),row=bs$row[i],nrows=bs$nrows[i]),
                  getValues(raster("m_B5.tif"),row=bs$row[i],nrows=bs$nrows[i]),
                  getValues(raster("m_B6.tif"),row=bs$row[i],nrows=bs$nrows[i]),
                  getValues(raster("m_B7.tif"),row=bs$row[i],nrows=bs$nrows[i]),
                  getValues(raster("TPI.tif"),row=bs$row[i],nrows=bs$nrows[i]),
                  getValues(raster("DEM.tif"),row=bs$row[i],nrows=bs$nrows[i]))
    
    for (v in 1:ndim){   # For each column (columns=Landsat Bands, TPI, DEM) we rescale by the mean and sd for that variable
      CNew[,v] <- (CNew[,v]-means[v])/sd[v]  # rescale (subtracting mean and dividing by sd)
    }
    
    Vals <- array(NA,dim=c(sims,dim(CNew)[1],nclass)) # Create empty array to store class probabilities for each posterior sample
    Ref <- matrix(NA,nrow=dim(CNew)[1], ncol=nclass)  # Create empty matrix to store most likely class for each posterior sample
    
    for (j in 1:nclass){
      Ref[,j] <- dmvnorm(CNew, mean=ModMean[j,], sigma=ModVar[j,,])
      
      for (s in 1:sims){
        pick <- sample(1:dim(SimsMean)[1],size=1)
        Vals[s,,j] <- dmvnorm(CNew, mean=SimsMean[pick,j,], sigma=SimsVar[pick,j,,])
      }
    }
    
    Ref2 <- unlist(apply(Ref, 1, function(x) ifelse(NA %in% x, NA, max(which(x==max(x))))))

    Final <- rep(0,times=dim(CNew)[1])
    for (s in 1:sims){
      Vals2 <- unlist(apply(Vals[s,,], 1, function(x) ifelse(NA %in% x, NA, max(which(x==max(x))))))
      Eval <- (Vals2==Ref2)
      Final <- Final+Eval
    }
    
    Final <- Final/sims
    out <- writeValues(out, Final, bs$row[i])
  }
  out <- writeStop(out)
  return(out)
}

system.time(s <- MakeUncertaintyLayer(raster("m_B2.tif"), mean=ModMean, sigma=ModVar, sims=5, filename=paste0(OutFolder,"/LC_test.tif")))

s2 <- focal(s,w=matrix(1,nrow=7,ncol=7),fun=mean,na.rm=TRUE)
