
##############################################################################################################
# USER INPUTS NEEDED
##############################################################################################################

# Step 1: Give path to folder with Landsat-processing R scripts
ScriptFolder <- "C:/Users/Grant/Desktop/SI_Postdoc/TNI_LCmapping/Analysis/Scripts/"

# Step 2: Give path to folder for temporary/output rasters
OutFolder <- "D:/GIS/Myanmar/LSAT/Tanintharyi"

# Step 3: Give the filename for the DEM to use for topographic correction (same projection and resolution)
DEMFileName <- "D:/GIS/Myanmar/Elevation/tni_DEMSRTM_30m_NASA/SourceData/tni_DEMSRTM_30m_NASA_WGS84_utm.tif"

# Step 4: Specify base directory for Landsat imagery and an array giving folder paths to unzipped images
# Currently, this must be in the same format as below
baseDir <- "D:/GIS/Myanmar/LSAT"
imageDirArray <- matrix(c("129_52","LC81290522016071LGN00.tar","LC81290522016071LGN00",
                          "130_50","LC81300502016046LGN00.tar","LC81300502016046LGN00",
                          "130_51","LC81300512016046LGN00.tar","LC81300512016046LGN00",
                          "130_52","LC81300522016078LGN00.tar","LC81300522016078LGN00",
                          "130_53","LC81300532016078LGN00.tar","LC81300532016078LGN00",
                          "131_50","LC81310502016069LGN00.tar","LC81310502016069LGN00",
                          "131_51","LC81310512016069LGN00.tar","LC81310512016069LGN00",
                          "131_52","LC81310522016069LGN00.tar","LC81310522016069LGN00"),ncol=3,byrow=T)

# Step 5: Specify 2-column matrix giving tile mosaic order.  The first column gives the tile to apply gamma correction to.
# The second column gives the reference tile to match.  After gamma correction, overlapping regions of adjacent tiles are
# averaged during mosaicking.
gcr <- matrix(c(5,4,
                1,4,
                3,4,
                8,4,
                2,3,
                6,2,
                7,3),ncol=2,byrow=T)

# Optional - specify location of temporary directory if looping over Landsat tiles causes memory issues
tmpDIR <- "D:/temp/"
rasterOptions(tmpdir=tmpDIR)

##############################################################################################################
# PRE-PROCESSING OF LANDSAT IMAGERY (TOA CONVERSION, TOPO CORRECTION, CLOUD REMOVAL, SAVE, MOSAIC)
##############################################################################################################

BANDS <- c(2:7,9:11)  # Do not change - All these bands needed for cloud identification script

# Load necessary libraries (Must have SAGA installed for cloud shadow removal)
library(raster)
library(landsat)
library(RSAGA)

for (TILE in 1:nrow(imageDirArray)){

  cat("Processing image", imageDirArray[TILE,1], "\n")
  
  # Set the working directory
  setwd(paste0(baseDir,"/",imageDirArray[TILE,1],"/",imageDirArray[TILE,2],"/",imageDirArray[TILE,3]))

  # Run radiance to Top-of-Atmosphere conversion script
  source(file = paste0(ScriptFolder,"TOA_correction.R"))
  
  # Run cloud removal script
  source(file = paste0(ScriptFolder,"CloudRemoval.R"))
  
  # Run cloud shadow removal script
  system.time(source(file = paste0(ScriptFolder,"CloudShadowRemoval.R")))
  
  # Run topographic correction script
  source(file = paste0(ScriptFolder,"topo_correction.R"))
  
  # Optional - for many scenes may need to clear temporary folder
  removeTmpFiles(h=0)
}

# Loop over bands and mosaic across tiles using normalization script  ---  ALL TILES MUST BE IN SAME PROJECTION
source(file = paste0(ScriptFolder,"Gamma_Correction_Auto.R"))

# Calculate topographic position index (TPI) for classification
DEM <- TPI <- raster(DEMFileName)        # Import DEM
weights <- matrix(1/1088, nr=33,nc=33)   # Create weight matrix (defines neighborhood size as 33x33 pixels and equal contribution)
weights[17,17] <- 0                      # Replace center cell with 0 (focal cell not included among neighbors)
TPI[] <- DEM[]-focal(DEM, w=weights)[]   # TPI = Elevation - mean(Elevation of Neighbors)

# Resample to align with Landsat mosaics and write to output directory
TPI <- resample(TPI, raster("m_B2.tif"), method='bilinear')  # Realign raster with Landsat rasters
writeRaster(TPI, filename="TPI.tif", format="GTiff", overwrite=TRUE)   # write raster to file
DEM <- resample(DEM, raster("m_B2.tif"), method='bilinear')  # Realign raster with Landsat rasters
writeRaster(DEM, filename="DEM.tif", format="GTiff", overwrite=TRUE)   # write raster to file
